//  Beispiel 8 "Berechnung der Fließkomma-Rechenleistung ESP8266"
//  
//  Unter Verwendung der IoT Brick Library

#include <all_IoT.h>

float a = 0.0, b = 3.0, Zeit = 0.0;
long Start = 0, Ende = 0, Groesse = 1;

void setup()
{
  IoT_Init(true);             // Beinhaltet Serial.begin
  IoT_WatchDog(false);        // Verhindert, dass der ESP8266 sich während des Benchmarks resettet
  //Serial.begin(115200);     // Für Arduino Due etc., d.h. wenn nicht IoT_Init(true) aufgerufen wird
  IoT_TerminalWaitInit(true); // Auf Terminalverbindung warten, damit nichts verloren geht
  Serial.println("");
  Serial.print(chr(12));
  Serial.println("Willkommen zum IoT GFLOP Test" + spc(37) + "(c) 2015-2017");
  Serial.println(repeatstr("-", 79) + chr(13));
  Serial.print("CPU " + t_CPUname());
  Serial.print(" (ID " + str(t_CPUtype()) + "), ");
  Serial.print(str(t_CPUbits()) + " Bit, ");
  Serial.print(str(t_CPUflashRAM()) + " kB Flash-RAM, ");
  Serial.print(str(t_CPUstaticRAM()) + " kB SRAM, ");
  Serial.println(str(t_MemAvail() / 1024) + " kB frei." + chr(13));
  t_DateTime dt = t_CompileDateTime();        // Uhrzeit, wann der Sketch kompiliert wurde
  byte dayOfWeek = dayofweek(dt.year, dt.month, dt.day);
  Serial.print("Datum : " + datestr(dt));
  Serial.print(" (" + dayname(dayOfWeek) +")");
  Serial.print(spc(20));
  Serial.println("Uhrzeit : " + timestr(dt));
  Serial.println("");//....1....!....2....!....3....!....4....!....5....!....6....!....7....!....8
  Serial.println("Der GFLOP Test berechnet die Anzahl an Fliesskomma-Additionen, welche die CPU ");
  Serial.println("pro Sekunde berechnen kann. Mit jeder Zeile werden mehr Additionen berechnet, ");
  Serial.println("wodurch eine hoehere Genauigkeit erreicht wird. Maximum sind 100 Mio. Fliess- ");
  Serial.println("komma-Additionen. Arduino Nano, Uno und Mega 2560 berechnen ca. 0,1 Mio. Addi-");
  Serial.println("tionen pro Sekunde, der ESP8266 berechnet ca. 1,4 Mio. Additionen pro Sekunde,");
  Serial.println("der ESP 32 kommt dank intergrierter FPU auf 29,7 Mio. Additionen pro Sekunde.");
  Serial.println("");
  while (Zeit < 3000000)
  {
    Groesse = Groesse * 10;
    Start = micros();
    a = 0.0;
    for (long i = 1; i <= Groesse; i++)
    {
        a = a + b;
    }
    Ende = micros();
    Zeit = Ende - Start;
    if (Zeit > 0)
    {
      Serial.print(strform(Groesse, 32, 1, true));
      Serial.print(" Schritte - benoetigte Zeit = ");
      Serial.print(strform(Zeit / 1000, 32, 1, true));
      Serial.print(" ms. = ");
      Serial.print(strform(Groesse * 1000.0 / Zeit * 1000.0, 32, 1, true));
      Serial.println(" FLOPs.");
      delay(10);
    }
  }
  IoT_Idle();           // Verhindert, dass der ESP8266 sich resettet
  String S = String(a); // Verhindert, dass die for-Schleife wegoptimiert wird
  Serial.println("");
  Serial.println("Fertig.");
}

void loop()
{
  IoT_Idle();           // Verhindert, dass der ESP8266 sich resettet
  delay(1000);
}

// Arduino Nano ( 16 MHz,  8 Bit) =    104.024 FLOPs (  1 Mio. Schritte)
// Arduino Mega ( 16 MHz,  8 Bit) =    100.717 FLOPs (  1 Mio. Schritte)
// Arduino Due  ( 84 MHz, 32 Bit) =    822.665 FLOPs ( 10 Mio. Schritte)
// Teensy 3.1   ( 96 MHz, 32 Bit) =  1.408.183 FLOPs ( 10 Mio. Schritte)
// Teensy 3.1   (120 MHz, 32 Bit) =  1.709.976 FLOPs ( 10 Mio. Schritte)
// Teensy 3.1   (144 MHz, 32 Bit) =  1.690.433 FLOPs ( 10 Mio. Schritte)
// ESP 8266     ( 80 MHz, 32 Bit) =  1.428.530 FLOPs (  1 Mio. Schritte)
// ESP 32       (240 MHz, 32 Bit) = 29.799.535 FLOPs (100 Mio. Schritte)

// Freie Bytes wenn keine Libraries geladen werden

// Arduino Nano (  16 MHz,  8 Bit) =     805 Bytes frei (von   2 kB)
// Arduino Mega (  16 MHz,  8 Bit) =   7.964 Bytes frei (von   8 kB)
// Arduino Due  (  84 MHz, 32 Bit) =  93.403 Bytes frei (von  96 kB)
// Teensy 3.1   (  96 MHz, 32 Bit) =  60.584 Bytes frei (von  64 kB)
// ESP 8266     (  80 MHz, 32 Bit) =  48.456 Bytes frei (von  96 kB)
// ESP 32       ( 240 MHz, 32 Bit) = 210.804 Bytes frei (von 520 kB)

